{-# LANGUAGE NoImplicitPrelude #-}
module Foreign.ForeignPtr.Unsafe.Compat (
  -- ** Unsafe low-level operations
  unsafeForeignPtrToPtr
) where

import Foreign.ForeignPtr.Unsafe (unsafeForeignPtrToPtr)
